/*********************************************************************
 * Job Ticket API (JTAPI)                                            
 *                                                                   
 * Copyright (c) 2003 International Business Machines Corp.  
 * Copyright (c) 2003 Free Standards Group.                   
 *                                                                    
 *                    Common Public License - v 1.0 
 * THE ACCOMPANYING PROGRAM IS PROVIDED UNDER THE TERMS OF THIS COMMON 
 * PUBLIC LICENSE ("AGREEMENT"). ANY USE, REPRODUCTION OR DISTRIBUTION 
 * OF THE PROGRAM CONSTITUTES RECIPIENT'S ACCEPTANCE OF THIS AGREEMENT.   
 *
 * The Common Public License in its entirety is located in 
 * jtapi_cpl.h and applies to the contents of this file. 
 *********************************************************************/

#ifndef JTAPI_H
#define JTAPI_H

#include <jtapi_cpl.h>

/************************************************************
 * Types used by the jtapi
 ***********************************************************/
typedef struct jtapi_int_object_s* jtapi_object_t;
typedef struct jtapi_int_attr_s* jtapi_attribute_t;

/************************************************************ 
 * Error codes produced by the jtapi
 ***********************************************************/
typedef enum
{
   JTAPI_OK = 0,
   JTAPI_INTERNAL_ERROR,     /* Unexpected error occured, such
                                as a memory allocation failure. */
   JTAPI_INVALID_REQUEST,    /* Request is not valid. */
   JTAPI_NOT_SUPPORTED,      /* Operation is not supported by 
                                the jtapi. */
   JTAPI_IO_ERROR,           /* Error occured while attempting
                                to read/write a job ticket file. */
   PARSER_EMPTY_JT,          /* The provided job ticket contains
                                no data. */
   PARSER_ERROR_AT_LINE,     /* Parser error at a specific line 
                                in the job ticket file. If a line 
                                number is available, the error will  
                                be reported instead of one of the
                                other more general error codes. */
   PARSER_FAILED,            /* Parsing the job ticket failed for  
                                an unknown reason. */
   PARSER_INIT_ERROR,        /* Error occurred during parser 
                                initialization */
   PARSER_INVALID_PARAM,     /* A function parameter to the parser 
                                code was not as expected. */
   PARSER_MISSING_INFO,      /* Information required to complete 
                                a parser function was not provided 
                                by the caller. */  
   PARSER_MISSING_PARAM,     /* A parameter that is required was not
                                provided by the caller. */    
   PARSER_UNSUPPORTED_JT_VERSION,   /* Job ticket version is not 
                                      supported. */
   PARSER_UNSUPPORTED_XML_VERSION,  /* Unsupported XML version. */
   PARSER_UNSUPPORTED_XML_ENCODING, /* Unsupported XML encoding. */
} jtapi_status_t;

/******************************************************************
* Structure that contains the status and additional information if
* a parser error occured.
******************************************************************/
typedef struct parser_status_s
{
   jtapi_status_t returnCode;  /* Return code */
   int lineNumber;             /* Line number where error 
                                * occured or 0 if unavailable. */
   int columnNumber;           /* Column number where 
                                * error occured or 0 if 
                                * unavailable.*/
} parser_status_t;

/************************************************************
 * The following functions apply to all of the jtapi objects
 ***********************************************************/

/************************************************************
 * destroy - Free the memory used by the provided object.
 *
 * param object - Pointer to the object to free 
 ***********************************************************/
void destroy(jtapi_object_t object);

/************************************************************
 * get - Gets the attribute having the provided name.
 *
 * param object - Pointer to the object from which to get
 *                the attribute
 * param attribute - Name of the attribute to get
 *
 * return - jtapi_attribute_t - Attribute having provided
 *      name
 ***********************************************************/
jtapi_attribute_t get(jtapi_object_t object, char* attribute);

/************************************************************
 * set - Sets the provided attribute for the provided object.
 *
 * param object - Pointer to the object in which to set
 *     the attribute
 * param attribute - Attribute to set
 *
 * return - jtapi_status_t JTAPI_OK if the call completed 
 *     successfully
 ***********************************************************/
jtapi_status_t set(jtapi_object_t object, 
                   jtapi_attribute_t attribute);

/************************************************************
 * setDoubleAttribute - Convenience function for setting a 
 * double attribute to the provided value. This function 
 * creates a jtapi_attribute_t and calls the set() function 
 * and then destroys the attribute.
 *
 * param jtapi_object_t object - Set the attribute for this
 *       object
 * param char* attribute - Name of the attribute to set
 * param double value - New value for the attribute
 *
 * return jtapi_status_t JTAPI_OK if the operation completed
 *       successfully
 ************************************************************/
jtapi_status_t setDoubleAttribute(jtapi_object_t object,
                                  char* attribute,
                                  double value);

/************************************************************
 * setIntegerAttribute - Convenience function for setting an 
 * integer attribute to the provided value. This function 
 * creates a jtapi_attribute_t and calls the set() function 
 * and then destroys the attribute.
 *
 * param jtapi_object_t object - Set the attribute for this
 *       object
 * param char* attribute - Name of the attribute to set
 * param int value - New value for the attribute
 *
 * return jtapi_status_t JTAPI_OK if the operation completed
 *       successfully
 ************************************************************/
jtapi_status_t setIntegerAttribute(jtapi_object_t object,
                                   char* attribute,
                                   int value);

/************************************************************
 * setObjectAttribute - Convenience function for setting an 
 * object attribute to the provided value. This function 
 * creates a jtapi_attribute_t and calls the set() function 
 * and then destroys the attribute.
 *
 * param jtapi_object_t object - Set the attribute for this
 *       object
 * param char* attribute - Name of the attribute to set
 * param jtapi_object_t value - New value for the attribute
 *
 * return jtapi_status_t JTAPI_OK if the operation completed
 *       successfully
 ************************************************************/
jtapi_status_t setObjectAttribute(jtapi_object_t object,
                                  char* attribute,
                                  jtapi_object_t value);

/************************************************************
 * setObjectsAttribute - Convenience function for setting an 
 * object attribute to the provided list of values. This 
 * function creates a jtapi_attribute_t adds the values and 
 * calls the set() function and then destroys the attribute.
 *
 * param jtapi_object_t object - Set the attribute for this
 *       object
 * param char* attribute - Name of the attribute to set
 * param jtapi_object_t* value - List of new values for 
 *       the attribute
 *
 * return jtapi_status_t JTAPI_OK if the operation completed
 *       successfully
 ************************************************************/
jtapi_status_t setObjectsAttribute(jtapi_object_t object,
                                   char* attribute,
                                   jtapi_object_t* list);

/************************************************************
 * setStringAttribute - Convenience function for setting a 
 * string attribute to the provided value. This function 
 * creates a jtapi_attribute_t calls the set() function and 
 * then destroys the attribute.
 *
 * param jtapi_object_t object - Set the attribute for this
 *       object
 * param char* attribute - Name of the attribute to set
 * param char* value - New value for the attribute
 *
 * return jtapi_status_t JTAPI_OK if the operation completed
 *       successfully
 ************************************************************/
jtapi_status_t setStringAttribute(jtapi_object_t object,
                                  char* attribute,
                                  char* value);

/************************************************************
 * setStringsAttribute - Convenience function for setting a 
 * string attribute to the provided values. This function 
 * creates a jtapi_attribute_t calls the set() function and 
 * then destroys the attribute.
 *
 * param jtapi_object_t object - Set the attribute for this
 *       object
 * param char* attribute - Name of the attribute to set
 * param char** values - List of new values for the attribute
 *
 * return jtapi_status_t JTAPI_OK if the operation completed
 *       successfully
 ************************************************************/
jtapi_status_t setStringsAttribute(jtapi_object_t object,
                                  char* attribute,
                                  char** values);

#endif /* JTAPI_H */